package com.bomostory.sceneeditmodule.basicdata

import android.content.ContentResolver
import android.media.MediaMetadataRetriever
import android.os.Environment
import android.provider.MediaStore
import org.json.JSONObject
import java.io.File

object DataParser {

    fun parseMusicSource(contentResolver: ContentResolver,libraryPaths:List<String>):MusicSource{
        val musicSource = MusicSource()
        musicSource.library = parseMusic(libraryPaths)
        musicSource.device = parseMusic(contentResolver,libraryPaths)
        return musicSource
    }

    private fun parseMusic(paths: List<String>): ArrayList<Music> {
        val mediaMetadataRetriever = MediaMetadataRetriever()
        val musics = ArrayList<Music>()

        for (path in paths) {
            val file = File(path)
            mediaMetadataRetriever.setDataSource(path)

            val music = Music()
            music.name = file.name
            music.path = path
            music.duration = mediaMetadataRetriever.extractMetadata(MediaMetadataRetriever.METADATA_KEY_DURATION).toLong()
            musics.add(music)
        }
        return musics
    }

    private fun parseMusic(contentResolver: ContentResolver,paths: List<String>): ArrayList<Music>{
        val selection = MediaStore.Audio.Media.IS_MUSIC + " != 0"
        val projection = arrayOf(
                MediaStore.Audio.Media.TITLE,
                MediaStore.Audio.Media.DATA,
                MediaStore.Audio.Media.DURATION
        )
        val cursor = contentResolver.query(
                MediaStore.Audio.Media.EXTERNAL_CONTENT_URI,
                projection,
                selection,
                null, null
        )

        val musics = ArrayList<Music>()
        while (cursor.moveToNext()) {
            val music = Music()
            music.name = cursor.getString(0)
            music.path = cursor.getString(1)
            music.duration = cursor.getString(2).toLong()
            val file = File(music.path)
            if (!paths.contains(file.path)) {
                musics.add(music)
            }
        }
        return musics
    }
}