package com.bomostory.sceneeditmodule.utils

import android.media.MediaPlayer
import com.bomostory.sceneeditmodule.basicdata.Music
import java.util.*
import kotlin.collections.ArrayList

class MusicPlayer {
    interface OnPlayChangedListener {
        fun onPlayChanged(music: Music, position: Int)
    }

    private var musics = ArrayList<Music>()
    private var mediaPlayers = ArrayList<MediaPlayer>()

    private var isPause = false

    private var timer: Timer? = null
    var playChangedListener: OnPlayChangedListener? = null

    fun add(music: Music) {
        musics.add(music)
    }

    fun remove(music: Music) {
        musics.remove(music)
    }

    fun clear() {
        musics.clear()
    }

    fun play() {
        if (!isPause) {
            prepare()
            for (mediaPlayer in mediaPlayers) {
                mediaPlayer.apply {
                    prepareAsync()
                }
            }
        } else {
            isPause = false
            start()
        }
        startTimer()
    }

    fun play(position: Int) {
        if (!isPause) {
            prepare(position)

            for (mediaPlayer in mediaPlayers) {
                mediaPlayer.apply {
                    prepareAsync()
                }
            }
        } else {
            isPause = false
            seekTo(position)
            start()
        }
        startTimer()
    }

    private fun prepare() {
        mediaPlayers.clear()

        for (music in musics) {
            val mediaPlayer = MediaPlayer()
            mediaPlayer.apply {
                reset()
                setDataSource(music.path)
                setOnPreparedListener {
                    it.start()
                }
                setOnCompletionListener {
                    mediaPlayers.remove(it)
                }
                mediaPlayers.add(mediaPlayer)
            }
        }
    }

    private fun prepare(position: Int) {
        mediaPlayers.clear()

        for (music in musics) {
            val mediaPlayer = MediaPlayer()
            mediaPlayer.apply {
                reset()
                setDataSource(music.path)
                setOnPreparedListener {
                    it.seekTo(position)
                    it.start()
                }
                setOnCompletionListener {
                    mediaPlayers.remove(it)
                }
                mediaPlayers.add(mediaPlayer)
            }
        }
    }


    private fun start() {
        for (mediaPlayer in mediaPlayers) {
            mediaPlayer.apply {
                start()
            }
        }
    }

    fun pause() {
        isPause = true
        for (mediaPlayer in mediaPlayers) {
            mediaPlayer.pause()
        }
        timer?.cancel()
    }

    fun seekTo(mSec: Int) {
        for (mediaPlayer in mediaPlayers) {
            mediaPlayer.seekTo(mSec)
        }
    }

    fun setVolume(volume: Float) {
        for (mediaPlayer in mediaPlayers) {
            mediaPlayer.setVolume(volume, volume)
        }
    }

    fun stop() {
        isPause = false
        for (mediaPlayer in mediaPlayers) {
            mediaPlayer.apply {
                stop()
            }
        }
        timer?.cancel()
    }

    fun release() {
        for (mediaPlayer in mediaPlayers) {
            mediaPlayer.apply {
                release()
            }
        }
        mediaPlayers.clear()
    }

    private fun startTimer() {
        timer = Timer()
        timer?.schedule(object : TimerTask() {
            override fun run() {
                for (mediaPlayer in mediaPlayers) {
                    playChangedListener?.onPlayChanged(musics[mediaPlayers.indexOf(mediaPlayer)], mediaPlayer.currentPosition)
                }
            }
        }, 0, 1000)
    }
}